@@ -25,7 +25,7 @@ except ImportError:
     import z0lib
 
 
-__version__ = "2.0.3"
+__version__ = "2.0.4"
 python_version = "%s.%s" % (sys.version_info[0], sys.version_info[1])
 
 #
@@ -33,7 +33,7 @@ python_version = "%s.%s" % (sys.version_info[0], sys.version_info[1])
 # - requests: oca-maintainers-tools -> '==2.3.0',
 #             codecov -> '>=2.7.9'
 # Here we assume: Odoo 11.0 use python 3.5, Odoo 12.0 uses python 3.7
-# If version is 2.7 or 3.5 or 3.6 or 3.7 or 3.8 then it refers to python version
+# If version is 2.7 or 3.5 or 3.6 or 3.7 or 3.8 or 3.9 then it refers to python version
 REQVERSION = {
     "acme_tiny": {"6.1": ">=4.0.3"},
     "argparse": {"0": "==1.2.1"},
@@ -42,11 +42,10 @@ REQVERSION = {
     "Babel": {"6.1": "==1.3", "8.0": "==2.3.4"},
     "beautifulsoup": {"6.1": "==3.2.1"},
     "codicefiscale": {"6.1": "==0.9"},
-    # "coverage": {"2.7": "<5.0.0", "3.5": ">=5.0.0"},
     "coverage": {"2.7": "<5.6.0", "3.5": ">=5.0.0"},
     "cryptography": {"2.7": ">=2.2.2", "3.7": ">=38.0"},
     "decorator": {"6.1": "==3.4.0", "10.0": "==4.0.10"},
-    "docutils": {"6.1": "==0.12", "0": "==0.14"},  # Version by test pkgs
+    "docutils": {"6.1": "==0.12", "0": "==0.14", "3.7": "0.16"},        # By test pkgs
     "ebaysdk": {"6.1": "==2.1.4"},
     "ERPpeek": {"0": "==1.6.1"},
     "feedparser": {"6.1": "==5.1.3", "10.0": "==5.2.1"},
@@ -55,8 +54,8 @@ REQVERSION = {
     "gevent": {
         "6.1": "==1.0.1",
         "7.0": "==1.0.2",
-        "10.0": "==1.1.2",
-        "3.7": "==1.3.4",
+        "10.0": ">=1.1.2<=1.4.0",       # trying to test
+        "3.7": "==1.5.0",
     },
     "greenlet": {
         "6.1": "==0.4.2",
@@ -94,9 +93,25 @@ REQVERSION = {
     },
     "pycodestyle": {"0": "==2.3.1"},
     "pydot": {"6.1": "==1.0.2", "8.0": "==1.2.3"},
-    "Pygments": {"6.1": "==2.0.2", "0": "==2.2"},  # Version by test pkgs
-    "pylint": {"2.7": "==1.9.3", "3.5": "==2.3.0"},
-    "pylint-plugin-utils": {"2.7": "==0.4", "3.5": "==0.5"},
+    "Pygments": {
+        "2.7": "<2.6.0",
+        "3.5": ">=2.7.0"
+    },
+    "pylint": {
+        "2.7": "<2.0.0",
+        "3.5": "<2.7.0",
+        "3.6": "<2.14.0",
+        "3.7": ">=2.14.0",
+    },
+    "pylint-odoo": {
+        "2.7": "==3.5.0",
+        "3.5": ">=3.5.0",
+    },
+    "pylint-plugin-utils": {
+        "2.7": "==0.2.6",
+        "3.5": "==0.5",
+        "3.6": ">=0.7",
+    },
     "pyopenssl": {"0": ">=16.2.0"},  # by MQT
     "pyotp": {"2.7": "==2.3.0", "3.5": ">=2.4.0"},
     "pyPDF2": {"2.7": "==1.28.4", "3.5": ">=2.0", "3.6": ">=2.0", "3.7": ">=2.0"},
@@ -120,6 +135,7 @@ REQVERSION = {
     "PyWebDAV": {"6.1": "<0.9.8"},
     "PyYAML": {"6.1": "==3.11", "8.0": "==3.12", "3.7": "==3.13"},
     "qrcode": {"6.1": "==5.0.1", "7.0": "==5.1", "10.0": "==5.3"},
+    "readme-renderer" : {"2.7": "<25.0", "3.5": "<29.0", "3.6": ">=30.0"},
     "restructuredtext_lint": {"6.1": "==0.12.2", "0": "==1.1.3"},
     "reportlab": {"6.1": "==3.1.44", "10.0": "==3.3.0"},
     "requests": {"6.1": "==2.6.0", "10.0": "==2.11.1"},
@@ -155,7 +171,6 @@ ALIAS = {
     "gitpython": "GitPython",
     "jinja2": "Jinja2",
     "ldap": "python-ldap",
-    # "lxml": "lxml",
     "mako": "Mako",
     "markupsafe": "MarkupSafe",
     "openid": "python-openid",
@@ -191,6 +206,7 @@ ALIAS3 = {
     "python-dev": "python3-dev",
 }
 FORCE_ALIAS = {"docutils==0.12": "docutils==0.14"}
+FORCE_ALIAS3 = {"docutils==0.12": "docutils==0.16"}
 PIP_SECURE_PACKAGES = [
     "urllib3[secure]",
     "cryptography",
@@ -224,10 +240,8 @@ PIP_TEST_PACKAGES = [
     "Pygments",
     "pylint",
     "pylint-mccabe",
-    "pylint_odoo",
+    "pylint-odoo",
     "pylint-plugin-utils",
-    # "pyopenssl",
-    # "python_plus",
     "python-magic",
     "pyserial",
     "pytest",
@@ -239,7 +253,50 @@ PIP_TEST_PACKAGES = [
     "setuptools",
     "simplejson",
     "unittest2",
-    # "urllib3[secure]",
+    "websocket-client",
+    "whichcraft",
+    "wrapt",
+    "z0bug_odoo",
+    "zerobug",
+]
+PIP3_TEST_PACKAGES = [
+    "astroid",
+    "Click",
+    "configparser",
+    "codecov",
+    "coverage",
+    "coveralls",
+    "docopt",
+    "docutils",
+    "flake8",
+    "GitPython",
+    "isort",
+    "lazy_object_proxy",
+    "lxml",
+    "MarkupSafe",
+    "mock",
+    "pbr",
+    "polib",
+    "pycodestyle",
+    "pycparser",
+    "pyflakes",
+    "Pygments",
+    "pylint",
+    "pylint-mccabe",
+    "pylint-odoo",
+    "pylint-plugin-utils",
+    "python-magic",
+    "pyserial",
+    "pytest",
+    "PyWebDAV",
+    "PyYAML",
+    "QUnitSuite",
+    "restructuredtext_lint",
+    "rfc3986",
+    "setuptools",
+    "simplejson",
+    "translators",
+    "unittest2",
     "websocket-client",
     "whichcraft",
     "wrapt",
@@ -257,6 +314,36 @@ BIN_TEST_PACKAGES = [
 ]
 RPC_PACKAGES = ["clodoo", "odoorpc", "oerplib", "os0"]
 PIP_BASE_PACKAGES = [
+    "Babel",
+    "chardet",
+    "configparser",
+    "decorator",
+    "docutils",
+    "feedparser",
+    "future",
+    "gdata",
+    "gevent",
+    "html2text",
+    "Jinja2",
+    "lxml",
+    "Mako",
+    "numpy",
+    "passlib",
+    "psutil",
+    "psycogreen",
+    "psycopg2-binary",
+    "python-ldap",
+    "python-dateutil",
+    "python-openid",
+    "python-plus",
+    "pydot",
+    "pyparsing",
+    "simplejson",
+    "six",
+    "stdnum",
+    "vatnumber",
+]
+PIP_ODOO_BASE_PACKAGES = [
     "Babel",
     "chardet",
     "configparser",
@@ -276,7 +363,6 @@ PIP_BASE_PACKAGES = [
     "Pillow",
     "psutil",
     "psycogreen",
-    # 'psycopg2',
     "psycopg2-binary",
     "Python-Chart",
     "python-ldap",
@@ -292,7 +378,6 @@ PIP_BASE_PACKAGES = [
     "simplejson",
     "six",
     "stdnum",
-    # "urllib3[secure]",
     "vatnumber",
     "Werkzeug",
 ]
@@ -366,11 +451,11 @@ DEPS9 = [
     "docutils==0.16",
     "Pillow==3.4.1",
     "Pygments==2.0.2",
-    "pylint==1.9.3",
-    "pylint==1.9.5",
-    "pylint==2.3.0",
-    "pylint==2.5.3",
-    "pylint-plugin-utils==0.4",
+    # "pylint==1.9.3",
+    # "pylint==1.9.5",
+    # "pylint==2.3.0",
+    # "pylint==2.5.3",
+    # "pylint-plugin-utils==0.4",
     "six==1.15.0",
 ]
 
@@ -493,14 +578,14 @@ def name_n_version(full_item, with_version=None, odoo_ver=None, pyver=None):
             full_item = full_item.replace(item, ALIAS3[item])
             item = ALIAS3[item]
     defver = False
-    if with_version and not item_ver:
+    if odoo_ver and with_version and not item_ver:
         if item in REQVERSION:
             min_v = False
             valid_ver = False
             if pyver in REQVERSION[item]:
                 min_v = pyver
             elif pyver and pyver.startswith("3"):
-                for v in ("3.8", "3.7", "3.6", "3.5"):
+                for v in ("3.9", "3.8", "3.7", "3.6", "3.5"):
                     if v in REQVERSION[item]:
                         min_v = v
                         break
@@ -535,7 +620,10 @@ def name_n_version(full_item, with_version=None, odoo_ver=None, pyver=None):
     if full_item.startswith("'"):
         full_item = full_item[1:-1]
     full_item = re.sub(' *([<=>]+) *', r'\1', full_item.strip())
-    full_item = FORCE_ALIAS.get(full_item, full_item)
+    if pyver and pyver.startswith("3"):
+        full_item = FORCE_ALIAS3.get(full_item, full_item)
+    else:
+        full_item = FORCE_ALIAS.get(full_item, full_item)
     return item, full_item, defver
 
 
@@ -661,14 +749,25 @@ def package_from_list(
     deps_list, kw, pkg_list, with_version=None, odoo_ver=None, pyver=None
 ):
     for item in pkg_list:
-        deps_list = add_package(
-            deps_list,
-            kw,
-            item,
-            with_version=with_version,
-            odoo_ver=odoo_ver,
-            pyver=pyver,
-        )
+        if isinstance(item, (list, tuple)):
+            for sub in item:
+                deps_list = add_package(
+                    deps_list,
+                    kw,
+                    sub,
+                    with_version=with_version,
+                    odoo_ver=odoo_ver,
+                    pyver=pyver,
+                )
+        else:
+            deps_list = add_package(
+                deps_list,
+                kw,
+                item,
+                with_version=with_version,
+                odoo_ver=odoo_ver,
+                pyver=pyver,
+            )
     return deps_list
 
 
@@ -968,10 +1067,10 @@ def main(cli_args=None):
         PY3_DEV = "python%s-dev" % ctx["pyver"]
     if ctx["odoo_ver"] and not ctx["pyver"]:
         ctx = get_pyver(ctx)
-    elif not ctx["odoo_ver"] and ctx["pyver"]:
-        ctx = get_def_odoo_ver(ctx)
-    if not ctx["odoo_ver"]:
-        ctx["odoo_ver"] = "12.0"
+    # elif not ctx["odoo_ver"] and ctx["pyver"]:
+    #     ctx = get_def_odoo_ver(ctx)
+    # if not ctx["odoo_ver"]:
+    #     ctx["odoo_ver"] = "12.0"
     if ctx["out_file"]:
         ctx = set_def_outfile(ctx)
     if not ctx["odoo_dir"] and ctx["odoo_ver"]:
@@ -1043,7 +1142,8 @@ def main(cli_args=None):
         deps_list = package_from_list(
             deps_list,
             "python",
-            PIP_BASE_PACKAGES + PIP_SECURE_PACKAGES,
+            PIP_ODOO_BASE_PACKAGES + PIP_SECURE_PACKAGES
+            if ctx["odoo_ver"] else PIP_BASE_PACKAGES + PIP_SECURE_PACKAGES,
             with_version=ctx["with_version"],
             odoo_ver=ctx["odoo_ver"],
             pyver=ctx["pyver"],
@@ -1086,14 +1186,24 @@ def main(cli_args=None):
             pyver=ctx["pyver"],
         )
     if ctx["test_pkgs"]:
-        deps_list = package_from_list(
-            deps_list,
-            "python",
-            PIP_TEST_PACKAGES,
-            with_version=ctx["with_version"],
-            odoo_ver=ctx["odoo_ver"],
-            pyver=ctx["pyver"],
-        )
+        if ctx['pyver'] and int(ctx['pyver'].split('.')[0]) == 3:
+            deps_list = package_from_list(
+                deps_list,
+                "python",
+                PIP3_TEST_PACKAGES,
+                with_version=ctx["with_version"],
+                odoo_ver=ctx["odoo_ver"],
+                pyver=ctx["pyver"],
+            )
+        else:
+            deps_list = package_from_list(
+                deps_list,
+                "python",
+                PIP_TEST_PACKAGES,
+                with_version=ctx["with_version"],
+                odoo_ver=ctx["odoo_ver"],
+                pyver=ctx["pyver"],
+            )
         deps_list = package_from_list(
             deps_list,
             "bin",
